package it.eng.spago.cms.access;
  
public interface PermissionVerifier {
  	
	public void init(String name);
	
	/**
	 * Verify if a user can add a node to a path. The method use
	 * the set of functionalities of the user to verify if at least
	 * one of them permits to add the node.
	 *  
	 * @param path, path of the node father
	 * @param functs, functionalities set of the user 	
	 * @return boolean, if true the user can perform the operation 
	 * false otherwise 
	 */
	public boolean canAddNode(String path, String[] roles);
	
	/**
	 * Verify if a user can set properties to a path. The method use
	 * the set of functionalities of the user to verify if at least
	 * one of them permits to set properties.
	 *  
	 * @param path, path of the node to add properties
	 * @param functs, functionalities set of the user 	
	 * @return boolean, if true the user can perform the operation 
	 * false otherwise 
	 */
	public boolean canSetProperty(String path, String[] roles);
	
	/**
	 * Verify if a user can remove a node. The method use
	 * the set of functionalities of the user to verify if at least
	 * one of them permits to remove the node.
	 *  
	 * @param path, path of the node to remove
	 * @param functs, functionalities set of the user 	
	 * @return boolean, if true the user can perform the operation 
	 * false otherwise 
	 */
	public boolean canRemove(String path, String[] roles);
	
	/**
	 * Verify if a user can read information of a node. The method use
	 * the set of functionalities of the user to verify if at least
	 * one of them permits to read the node.
	 *  
	 * @param path, path of the node to read
	 * @param functs, functionalities set of the user 	
	 * @return boolean, if true the user can perform the operation 
	 * false otherwise 
	 */
	public boolean canRead(String path, String[] roles);
	
}
