package it.eng.spago.cms.exec.operations;

import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.cms.exceptions.OperationNotFoundException;
import it.eng.spago.cms.exceptions.ParametersReplacingException;
import it.eng.spago.cms.util.ParamSourceBean;
import it.eng.spago.cms.util.constants.ModuleConstants;
import it.eng.spago.cms.util.constants.OperationsConstants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Vector;

public class OperationBuilder {
	
			
	public static SetOperation buildSetOperation() throws BuildOperationException { return new SetOperation(); }
	public static SetOperation buildSetOperation(String name) throws OperationNotFoundException { 
		SourceBean operationDescriptor = getOperationDescriptor(name);
		return new SetOperation(operationDescriptor); 
	}
	public static SetOperation buildSetOperation(String name, Vector params) throws OperationNotFoundException, ParametersReplacingException { 
		SourceBean operationDescriptor = getOperationDescriptor(name, params);
		return new SetOperation(operationDescriptor);
	}
	
	
	
	public static GetOperation buildGetOperation() throws BuildOperationException { return new GetOperation(); }
	public static GetOperation buildGetOperation(String name) throws OperationNotFoundException { 
		SourceBean operationDescriptor = getOperationDescriptor(name);
		return new GetOperation(operationDescriptor); 
	}
	public static GetOperation buildGetOperation(String name, Vector params) throws OperationNotFoundException, ParametersReplacingException { 
		SourceBean operationDescriptor = getOperationDescriptor(name, params);
		return new GetOperation(operationDescriptor); 	
    }
	
	
	
	public static RestoreOperation buildRestoreOperation() throws BuildOperationException { return new RestoreOperation(); }
	public static RestoreOperation buildRestoreOperation(String name) throws OperationNotFoundException { 
		SourceBean operationDescriptor = getOperationDescriptor(name);
		return new RestoreOperation(operationDescriptor); 
	}
	public static RestoreOperation buildRestoreOperation(String name, Vector params) throws OperationNotFoundException, ParametersReplacingException { 
		SourceBean operationDescriptor = getOperationDescriptor(name, params);
		return new RestoreOperation(operationDescriptor); 
	}

	
	
	
	
	public static DeleteOperation buildDeleteOperation() throws BuildOperationException { return new DeleteOperation(); }
	public static DeleteOperation buildDeleteOperation(String path) throws BuildOperationException { return new DeleteOperation(path); }
	public static DeleteOperation buildDeleteOperation(String path, String version) throws BuildOperationException { return new DeleteOperation(path, version); }
	public static DeleteOperation buildDeleteOperationFromFile(String name) throws OperationNotFoundException { 
		SourceBean operationDescriptor = getOperationDescriptor(name);
		return new DeleteOperation(operationDescriptor);
	}
	public static DeleteOperation buildDeleteOperationFromFile(String name, Vector params) throws OperationNotFoundException, ParametersReplacingException { 
		SourceBean operationDescriptor = getOperationDescriptor(name, params);
		return new DeleteOperation(operationDescriptor); 
	}
	
	
	
	
	
	public static SearchOperation buildSearchOperation() throws BuildOperationException { return new SearchOperation(); }
	public static SearchOperation buildSearchOperation(String name) throws OperationNotFoundException { 
		SourceBean operationDescriptor = getOperationDescriptor(name);
		return new SearchOperation(operationDescriptor);
	}
	public static SearchOperation buildSearchOperation(String name, Vector params) throws OperationNotFoundException, ParametersReplacingException { 
		SourceBean operationDescriptor = getOperationDescriptor(name, params);
		return new SearchOperation(operationDescriptor);
	}
	public static SearchOperation buildSearchOperation(String name, Vector params, String[] paramsQuery) throws OperationNotFoundException, ParametersReplacingException { 
		SourceBean operationDescriptor = getOperationDescriptor(name, params, paramsQuery);
		return new SearchOperation(operationDescriptor);
	}
	public static SearchOperation buildSearchOperation(String name, String[] paramsQuery) throws OperationNotFoundException, ParametersReplacingException { 
		SourceBean operationDescriptor = getOperationDescriptor(name, paramsQuery);
		return new SearchOperation(operationDescriptor);
	}


    
	
	
	
	private static SourceBean getOperationDescriptor(String name)throws OperationNotFoundException {
		SourceBean operationDescriptor = null;
		ConfigSingleton configure = ConfigSingleton.getInstance();
		operationDescriptor = (SourceBean)configure.
		                       getFilteredSourceBeanAttribute(OperationsConstants.OPERATION_ENVELOPE, 
		                       		                          OperationsConstants.OPERATION_NAME, name);
		if(operationDescriptor == null) {
	           TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
	                               "OperationBuilder::getOperationDescriptor(String): name of the operation " +
	                               "not find inside the configuration file");
	           throw new OperationNotFoundException("OperationBuilder::getOperationDescriptor: name of the " +
	           		                                "operation not find inside the configuration file");
		}
		SourceBeanAttribute descriptionOp =(SourceBeanAttribute)operationDescriptor.
		                                    getContainedAttributes().get(1);
		operationDescriptor = (SourceBean)descriptionOp.getValue();
	    return operationDescriptor;
	}
	
	
	
	
	
	
	private static SourceBean getOperationDescriptor(String name, Vector params) 
	                   throws OperationNotFoundException, ParametersReplacingException {
		SourceBean operationDescriptor = getOperationDescriptor(name);
		ParamSourceBean paramOperationDescriptor = null;
		try {
			paramOperationDescriptor = new ParamSourceBean(operationDescriptor);
			paramOperationDescriptor.replaceParamToken("?", params);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "OperationBuilder::getOperationDescriptor(String, Vector): " +
                                "Error during the parameters replace operation", e);
			throw new ParametersReplacingException("OperationBuilder::getOperationDescriptor(String, Vector): " +
												   "Error during the parameters replace operation", e);
		}
		return (SourceBean)paramOperationDescriptor;
	}

	
	
	
	
	private static SourceBean getOperationDescriptor(String name, String[] paramsQuery) 
	                   throws OperationNotFoundException, ParametersReplacingException {
		SourceBean operationDescriptor = getOperationDescriptor(name);
		String query = (String)operationDescriptor.getAttribute(OperationsConstants.QUERY_ATTRIBUTE);
		query = replaceParamsQuery(query, paramsQuery);
		try {
			operationDescriptor.updAttribute(OperationsConstants.QUERY_ATTRIBUTE, query);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "OperationBuilder::getOperationDescriptor(String, String[]): " +
                                "Error during the parameters replace operation", e);
            throw new ParametersReplacingException("OperationBuilder::getOperationDescriptor(String, String[]): " +
								"Error during the parameters replace operation", e);
		}
		return operationDescriptor;
	}
	
	
	
	
	
	private static SourceBean getOperationDescriptor(String name, Vector params, String[] paramsQuery)
					   throws OperationNotFoundException, ParametersReplacingException {
		SourceBean operationDescriptor = getOperationDescriptor(name, paramsQuery);
		ParamSourceBean paramOperationDescriptor = null;
		try {
			paramOperationDescriptor = new ParamSourceBean(operationDescriptor);
			paramOperationDescriptor.replaceParamToken("?", params);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "OperationBuilder::getOperationDescriptor(String, Vector, String[]): " +
                                "Error during the parameters replace operation", e);
			throw new ParametersReplacingException("OperationBuilder::getOperationDescriptor(String, Vector, String[]): " +
												   "Error during the parameters replace operation", e);
		}
		return (SourceBean)paramOperationDescriptor;
	}
	
	
	
	
	private static String replaceParamsQuery(String query, String[] params) 
	               throws ParametersReplacingException {
		for(int i=0; i<params.length; i++)  {
			if(params[i] == null) {
				TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                    "OperationBuilder::replaceParamsQuery: " +
                                    "Parameter "+i+" is null");
				throw new ParametersReplacingException("OperationBuilder::replaceParamsQuery: " +
						            "Parameter "+i+" is null");
			}
			if(query.indexOf(OperationsConstants.QUERY_PLACEHOLDER) == -1 ) {
				TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                    "OperationBuilder::replaceParamsQuery: " +
                                    "the number of parameters and values is different");
				throw new ParametersReplacingException("OperationBuilder::replaceParamsQuery: " +
			                        "the number of parameters and values is different");
			}
			query = query.replaceFirst(OperationsConstants.QUERY_PLACEHOLDER, params[i]);
		}
		return query;
	}
	
	
	
}

