package it.eng.spago.cms.exec.operations;

import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.cms.util.constants.ModuleConstants;
import it.eng.spago.cms.util.constants.OperationsConstants;
import it.eng.spago.cms.util.constants.PropertiesConstants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Calendar;

/*
 	<OPERATION name="">
  		<SETOPERATION path="" type="(container/content)" cancelOldProperties="(true/flase)">
  		 	<CONTENT stream="" /> (optional)
  		 	<PROPERTIES> (optional)
  		 		<PROPERTY name="" type="STRING/DATE/LONG/DOUBLE/BOOLEAN/BINARY" >
  		 			<PROPERTYVALUE value=""/>
  		 			<PROPERTYVALUE value=""/>
  		 			....
  		 		</PROPERTY>
  		 		....
  		 	</PROPERTIES>
  		</SETOPERATION>
  	</OPERATION> 


*/
public class SetOperation implements Operation {

	public final static String TYPE_CONTENT = "content";
	public final static String TYPE_CONTAINER = "container";
	
	private SourceBean operationDescriptor = null;
		
	public SetOperation(SourceBean descriptor){	
		operationDescriptor = descriptor;
	}
	
	public SetOperation() throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(OperationsConstants.NAME_SET_OPERATION);
			operationDescriptor.setAttribute(OperationsConstants.PATH, "");
			operationDescriptor.setAttribute(OperationsConstants.TYPE, "");
			SourceBean contentSB = new SourceBean(OperationsConstants.CONTENTSB);
			contentSB.setAttribute(OperationsConstants.STREAMATTR, new ByteArrayInputStream(new byte[0]));
			operationDescriptor.setAttribute(OperationsConstants.OLDPROPERTIES, "false");
			SourceBean props = new SourceBean(PropertiesConstants.PROPERTIES);
			operationDescriptor.setAttribute(props);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::<init>(): Error during the creation of the set operation" +
                                "SourceBean");
			throw new BuildOperationException("SetOperation::<init>(): Error during the creation of the " +
					                          "set operation SourceBean", e);
		}
	}
	
	public void setPath(String path) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(OperationsConstants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setPath: Error during the path setting");
            throw new BuildOperationException("SetOperation::setPath: Error during the path setting", e);
		}
	}
	
	public void setContent(InputStream is) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(OperationsConstants.STREAM, is);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setContent: Error during the content setting");
            throw new BuildOperationException("SetOperation::setContent: Error during the content setting", e);
		}
	}
	
	public void setType(String type) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(OperationsConstants.TYPE, type);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setType: Error during the type setting");
            throw new BuildOperationException("SetOperation::setType: Error during the type setting", e);
		}
	}
	
	public void setEraseOldProperties(boolean erase) throws BuildOperationException {
		String eraseStr = "false";
		if(erase) {
			eraseStr = "true";
		}
		try {
			operationDescriptor.updAttribute(OperationsConstants.OLDPROPERTIES, eraseStr);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setEraseOldProperties: Error during the erase old properties setting");
            throw new BuildOperationException("SetOperation::etEraseOldProperties: Error during the erase old properties setting", e);
		}
	}
	
	
	public void setStringProperty(String name, String[] values) throws BuildOperationException {
		
		try{
			SourceBean props = (SourceBean)operationDescriptor.getAttribute(PropertiesConstants.PROPERTIES);
			
			SourceBean prop = new SourceBean(PropertiesConstants.PROPERTY);
			prop.setAttribute(PropertiesConstants.NAME_PROPERTY, name);
			prop.setAttribute(PropertiesConstants.TYPE_PROPERTY, PropertiesConstants.TYPE_STRING);
			
			for(int i=0; i<values.length; i++) {
				SourceBean propval = new SourceBean(PropertiesConstants.VALUE_PROPERTY_SB);
				propval.setAttribute(PropertiesConstants.VALUE_PROPERTY, values[i]);
				prop.setAttribute(propval);
			}
			
			props.setAttribute(prop);
		}catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setStringProperty: Error during the property setting");
            throw new BuildOperationException("SetOperation::setStringProperty: Error during the property setting", e);
		}
	}
	
	//TODO implements the missing method setXXXProperty into the class SetOperation
	
	public void setBinaryProperty(String name, InputStream[] values) {
		
	}
	
	public void setDateProperty(String name, Calendar[] values) {
		
	}
	
	public void setDoubleProperty(String name, double[] values) {
		
	}
	
	public void setLongProperty(String name, long[] values) {
		
	}
	
	public void setBooleanProperty(String name, boolean[] values) {
		
	}

	
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}

	
	//TODO control and complete the method isComplete into SetOperation class
	
	public boolean isComplete() {
		boolean control = true;
		String path = (String)operationDescriptor.getAttribute(OperationsConstants.PATH);
		if( (path==null) || (path.trim().equals("")) ) {
			control = false;
		}
		String type = (String)operationDescriptor.getAttribute(OperationsConstants.TYPE);
		if( (type==null) || (type.trim().equals("")) ) {
			control = false;
		} else {
			if( !type.equalsIgnoreCase(OperationsConstants.CONTAINER) && !type.equalsIgnoreCase(OperationsConstants.CONTENT)) {
				control = false;	
			}
		}
		return control;
	}

	
	
}
